/**************************************************************************************************
  Filename:       simpleGATTprofile_ota.c
  Revised:
  Revision:

  Description:    This file contains the Simple GATT profile sample GATT service
                  profile for use with the BLE sample application.


**************************************************************************************************/

/*********************************************************************
 * INCLUDES
 */

#include "CONFIG.h"
#include "gattprofile.h"
#include "stdint.h"
#include "ble_custom_config_service.h"

#include "CH57x_common.h"

/*********************************************************************
 * MACROS
 */

/*********************************************************************
 * CONSTANTS
 */

#define SERVAPP_NUM_ATTR_SUPPORTED        7

#define RAWPASS_TX_VALUE_HANDLE     2
//#define RAWPASS_RX_VALUE_HANDLE     2
/*********************************************************************
 * TYPEDEFS
 */



// ble_uart GATT Profile Service UUID
const static uint8_t ble_custom_config_ServiceUUID[ATT_UUID_SIZE] =
{0x9E, 0xCA, 0xDC, 0x24, 0x0E, 0xE5, 0xA9, 0xE0, 0x93, 0xF3, 0xA3, 0xB5, 0x01, 0x00, 0x40, 0x6F};


// Characteristic tx uuid
const static uint8_t ble_custom_config_CharUUID[ATT_UUID_SIZE] =
{0x9E, 0xCA, 0xDC, 0x24, 0x0E, 0xE5, 0xA9, 0xE0, 0x93, 0xF3, 0xA3, 0xB5, 0x03, 0x00, 0x40, 0x6F};

/*********************************************************************
 * EXTERNAL VARIABLES
 */

/*********************************************************************
 * EXTERNAL FUNCTIONS
 */

/*********************************************************************
 * LOCAL VARIABLES
 */

static ble_custom_config_ProfileChangeCB_t ble_uart_AppCBs = NULL;

/*********************************************************************
 * Profile Attributes - variables
 */

// Profile Service attribute
static CONST gattAttrType_t ble_uart_Service = { ATT_UUID_SIZE, ble_custom_config_ServiceUUID };




// Characteristic 1 Value
static uint8 ble_uart_RxCharValue[BLE_UART_RX_BUFF_SIZE];
//static uint8 ble_uart_RxCharValue[1];

// Profile Characteristic 2 Properties
//static uint8 ble_uart_TxCharProps = GATT_PROP_NOTIFY| GATT_PROP_INDICATE;
static uint8 ble_custom_config_service_CharProps = GATT_PROP_WRITE|GATT_PROP_READ;

// Characteristic 2 Value
static uint8 ble_uart_TxCharValue = 0;


/*********************************************************************
 * Profile Attributes - Table
 */

static gattAttribute_t ble_custom_config_service_ProfileAttrTbl[] =
{
    // Simple Profile Service
    {
        { ATT_BT_UUID_SIZE, primaryServiceUUID }, /* type */
        GATT_PERMIT_READ,                         /* permissions */
        0,                                        /* handle */
        (uint8 *)&ble_uart_Service            /* pValue */
    },
    // Characteristic 2 Declaration
    {
        { ATT_BT_UUID_SIZE, characterUUID },
        GATT_PERMIT_READ,
        0,
        &ble_custom_config_service_CharProps
    },

    // Characteristic Value 2
    {
        { ATT_UUID_SIZE, ble_custom_config_CharUUID },
        GATT_PERMIT_WRITE|GATT_PERMIT_READ,
        0,
        (uint8 *)&ble_uart_TxCharValue
    },


};


/*********************************************************************
 * LOCAL FUNCTIONS
 */
static bStatus_t ble_uart_ReadAttrCB( uint16 connHandle, gattAttribute_t *pAttr,
                                     uint8 *pValue, uint16 *pLen, uint16 offset, uint16 maxLen,uint8 method  );
static bStatus_t ble_uart_WriteAttrCB( uint16 connHandle, gattAttribute_t *pAttr,
                                      uint8 *pValue, uint16 len, uint16 offset,uint8 method );

static void ble_uart_HandleConnStatusCB ( uint16 connHandle, uint8 changeType );


/*********************************************************************
 * PROFILE CALLBACKS
 */
// Simple Profile Service Callbacks
static gattServiceCBs_t ble_uart_ProfileCBs =
{
    ble_uart_ReadAttrCB,  // Read callback function pointer
    ble_uart_WriteAttrCB, // Write callback function pointer
    NULL                       // Authorization callback function pointer
};

/*********************************************************************
 * PUBLIC FUNCTIONS
 */

/*********************************************************************
 * @fn      ble_uart_AddService
 *
 * @brief   Initializes the Simple Profile service by registering
 *          GATT attributes with the GATT server.
 *
 * @param   services - services to add. This is a bit map and can
 *                     contain more than one service.
 *
 * @return  Success or Failure
 */
bStatus_t ble_custom_config_service( ble_custom_config_ProfileChangeCB_t cb)
{
    uint8 status = SUCCESS;

    //GATTServApp_InitCharCfg( INVALID_CONNHANDLE, ble_uart_TxCCCD );
    // Register with Link DB to receive link status change callback
    linkDB_Register( ble_uart_HandleConnStatusCB  );

//    ble_uart_TxCCCD.connHandle = INVALID_CONNHANDLE;
//    ble_uart_TxCCCD.value = 0;
    // Register GATT attribute list and CBs with GATT Server App
    status = GATTServApp_RegisterService( ble_custom_config_service_ProfileAttrTbl,
                                          GATT_NUM_ATTRS( ble_custom_config_service_ProfileAttrTbl ),
                                          GATT_MAX_ENCRYPT_KEY_SIZE,
                                          &ble_uart_ProfileCBs );
    if(status!=SUCCESS)
        PRINT("Add ble uart service failed!\n");
    ble_uart_AppCBs = cb;

    return ( status );
}



/*********************************************************************
 * @fn          ble_uart_ReadAttrCB
 *
 * @brief       Read an attribute.
 *
 * @param       connHandle - connection message was received on
 * @param       pAttr - pointer to attribute
 * @param       pValue - pointer to data to be read
 * @param       pLen - length of data to be read
 * @param       offset - offset of the first octet to be read
 * @param       maxLen - maximum length of data to be read
 *
 * @return      Success or Failure
 */
static bStatus_t ble_uart_ReadAttrCB( uint16 connHandle, gattAttribute_t *pAttr,
                                     uint8 *pValue, uint16 *pLen, uint16 offset, uint16 maxLen,uint8 method )
{
    bStatus_t status = SUCCESS;
    PRINT("ReadAttrCB\n");
    // If attribute permissions require authorization to read, return error
    if ( gattPermitAuthorRead( pAttr->permissions ) )
    {
        // Insufficient authorization
        return ( ATT_ERR_INSUFFICIENT_AUTHOR );
    }

    // Make sure it's not a blob operation (no attributes in the profile are long)
    if ( pAttr->type.len == ATT_BT_UUID_SIZE )
    {
        // 16-bit UUID
        uint16 uuid = BUILD_UINT16( pAttr->type.uuid[0], pAttr->type.uuid[1]);
        if(uuid == GATT_CLIENT_CHAR_CFG_UUID)
        {
            *pLen = 2;
            tmos_memcpy(pValue, pAttr->pValue, 2);
        }
    }
    else
    {
        if(tmos_memcmp(pAttr->type.uuid, ble_custom_config_CharUUID, 16))
        {

                //PRINT("HEELOE \r\n");

                ble_custom_config_evt_t evt;
                evt.type = BLE_CUSTOM_CONFIG_READ;

                evt.data.length = pLen;
                evt.data.p_data = pValue;            
                ble_uart_AppCBs(connHandle,&evt);       
                
        }
//        else if(!tmos_memcmp(pAttr->type.uuid, ble_uart_RxCharUUID, 16))
//        {
//            PRINT("read tx char\n");
//        }
    }

    return ( status );
}

/*********************************************************************
 * @fn      simpleProfile_WriteAttrCB
 *
 * @brief   Validate attribute data prior to a write operation
 *
 * @param   connHandle - connection message was received on
 * @param   pAttr - pointer to attribute
 * @param   pValue - pointer to data to be written
 * @param   len - length of data
 * @param   offset - offset of the first octet to be written
 *
 * @return  Success or Failure
 */

static bStatus_t ble_uart_WriteAttrCB( uint16 connHandle, gattAttribute_t *pAttr,
                                      uint8 *pValue, uint16 len, uint16 offset,uint8 method )
{
    bStatus_t status = SUCCESS;
    //uint8 notifyApp = 0xFF;
    // If attribute permissions require authorization to write, return error
    if ( gattPermitAuthorWrite( pAttr->permissions ) )
    {
        // Insufficient authorization
        return ( ATT_ERR_INSUFFICIENT_AUTHOR );
    }

    if ( pAttr->type.len == ATT_BT_UUID_SIZE )
    {
        
#if 0
        // 16-bit UUID
        uint16 uuid = BUILD_UINT16( pAttr->type.uuid[0], pAttr->type.uuid[1]);
        if(uuid == GATT_CLIENT_CHAR_CFG_UUID)
        {
            status = GATTServApp_ProcessCCCWriteReq( connHandle, pAttr, pValue, len,
                     offset, GATT_CLIENT_CFG_NOTIFY );
            if ( status == SUCCESS && ble_uart_AppCBs)
            {
                uint16 charCfg = BUILD_UINT16( pValue[0], pValue[1] );
                ble_uart_evt_t evt;

                //printf("CCCD set: [%d]\n", charCfg);
                evt.type = (charCfg == GATT_CFG_NO_OPERATION)?BLE_UART_EVT_TX_NOTI_DISABLED:BLE_UART_EVT_TX_NOTI_ENABLED;
                ble_uart_AppCBs(connHandle,&evt);
            }
        }
#endif
        
    }
    else
    {
        // 128-bit UUID
        if(pAttr->handle == ble_custom_config_service_ProfileAttrTbl[RAWPASS_TX_VALUE_HANDLE].handle)
        {
            if(ble_uart_AppCBs) {
                uint16_t write_length = len;
                ble_custom_config_evt_t evt;
                evt.type = BLE_CUSTOM_CONFIG_WRITE;
                evt.data.length = &write_length;
                evt.data.p_data = pValue;
                ble_uart_AppCBs(connHandle,&evt);
            }
        }
    }
    return ( status );
}

/*********************************************************************
 * @fn          simpleProfile_HandleConnStatusCB
 *
 * @brief       Simple Profile link status change handler function.
 *
 * @param       connHandle - connection handle
 * @param       changeType - type of change
 *
 * @return      none
 */
static void ble_uart_HandleConnStatusCB ( uint16 connHandle, uint8 changeType )
{
    // Make sure this is not loopback connection
    if ( connHandle != LOOPBACK_CONNHANDLE )
    {
        // Reset Client Char Config if connection has dropped
        if ( ( changeType == LINKDB_STATUS_UPDATE_REMOVED )      ||
                ( ( changeType == LINKDB_STATUS_UPDATE_STATEFLAGS ) &&
                  ( !linkDB_Up( connHandle ) ) ) )
        {
            //ble_uart_TxCCCD[0].value = 0;
            //GATTServApp_InitCharCfg( connHandle, ble_uart_TxCCCD );
            //printf("clear client configuration\n");
        }
    }
}


/*********************************************************************
*********************************************************************/

